/*
Copyright (c) 2014-2022 Martin Cejp

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

/**
 * Copyright: Martin Cejp 2014-2022.
 * License: $(LINK2 boost.org/LICENSE_1_0.txt, Boost License 1.0).
 * Authors: Martin Cejp
 */
module dlib.filesystem.delegaterange;

private import std.range;

/**
 * An input range that enumerates items by obtaining them from a delegate until it returns 0
 */
class DelegateInputRange(T): InputRange!T
{
    bool delegate(out T t) fetch;
    bool have = false;
    T front_;

    this(bool delegate(out T t) fetch)
    {
        this.fetch = fetch;
    }

    override bool empty()
    {
        if (!have)
            have = fetch(front_);

        return !have;
    }

    override T front()
    {
        return front_;
    }

    override void popFront()
    {
        have = false;
    }

    override T moveFront()
    {
        have = false;
        return front_;
    }

    override int opApply(scope int delegate(T) dg)
    {
        int result = 0;

        for (size_t i = 0; !empty; i++)
        {
            result = dg(moveFront);

            if (result != 0)
                break;
        }

        return result;
    }

    override int opApply(scope int delegate(size_t, T) dg)
    {
        int result = 0;

        for (size_t i = 0; !empty; i++)
        {
            result = dg(i, moveFront);

            if (result != 0)
                break;
        }

        return result;
    }
}
