/*
Copyright (c) 2011-2022 Timur Gafarov

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

/**
 * Image sharpening
 *
 * Copyright: Timur Gafarov 2011-2022.
 * License: $(LINK2 boost.org/LICENSE_1_0.txt, Boost License 1.0).
 * Authors: Timur Gafarov
 */
module dlib.image.filters.sharpen;

import dlib.image.image;
import dlib.image.color;
import dlib.image.arithmetics;
import dlib.image.filters.contrast;
import dlib.image.filters.boxblur;

/// Sharpen an image
SuperImage sharpen(SuperImage src, SuperImage outp, int radius, float amount)
{
    if (outp is null)
        outp = src.dup;

    auto blurred = boxBlur(src, outp, radius);
    auto mask = subtract(src, blurred, outp, 1.0f);
    auto highcon = contrast(mask, outp, amount, ContrastMethod.AverageImage);
    return add(src, highcon, outp, 0.25f);
}

/// ditto
SuperImage sharpen(SuperImage src, int radius, float amount)
{
    return sharpen(src, null, radius, amount);
}
