/*
Copyright (c) 2013-2022 Timur Gafarov

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

/**
 * Linear algebra and numerical analysis
 *
 * Description:
 * dlib.math brings vector and matrix types to D, as well as some numerical 
 * analysis functionality. dlib.math is great as a math library for games, 
 * graphics/physics engines and rendering pipelines. All types are POD and 
 * OpenGL-friendly: you can pass your 4x4 matrices to OpenGL functions directly, 
 * without any conversion.
 *
 * Copyright: Timur Gafarov 2013-2022.
 * License: $(LINK2 boost.org/LICENSE_1_0.txt, Boost License 1.0).
 * Authors: Timur Gafarov
 */
module dlib.math;

public
{
    import dlib.math.combinatorics;
    import dlib.math.complex;
    import dlib.math.decomposition;
    import dlib.math.diff;
    import dlib.math.dual;
    import dlib.math.dualquaternion;
    import dlib.math.fft;
    import dlib.math.hof;
    import dlib.math.interpolation;
    import dlib.math.linsolve;
    import dlib.math.matrix;
    import dlib.math.quaternion;
    import dlib.math.sse;
    import dlib.math.tensor;
    import dlib.math.transformation;
    import dlib.math.utils;
    import dlib.math.vector;
}
