/*
Copyright (c) 2016-2022 Timur Gafarov

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

/**
 * Text processing utils
 *
 * Copyright: Timur Gafarov 2016-2022.
 * License: $(LINK2 boost.org/LICENSE_1_0.txt, Boost License 1.0).
 * Authors: Timur Gafarov
 */
module dlib.text.utils;

import dlib.core.memory;

/// Make an copy of a string in unmanaged memory
T[] copy(T)(T[] b)
{
    auto res = New!(T[])(b.length);
    foreach(i, c; b)
        res[i] = c;
    return res;
}

///
unittest
{
    auto str = "hello".dup;
    auto c = copy(str);
    assert(c == str);
    Delete(c);
}

/// Make an immutable copy of a string in unmanaged memory
immutable(T)[] immutableCopy(T)(immutable(T)[] b)
{
    auto res = New!(T[])(b.length);
    foreach(i, c; b)
        res[i] = c;
    return cast(immutable(T)[])res;
}

/// Concatenates two strings to a new string in unmanaged memory
string catStr(string s1, string s2)
{
    char[] buffer = New!(char[])(s1.length + s2.length);
    size_t i, j;
    for(i = 0; i < s1.length; i++)
    {
        buffer[i] = s1[i];
    }
    for(j = 0; j < s2.length; j++)
    {
        buffer[i+j] = s2[j];
    }
    return cast(string)buffer;
}

///
unittest
{
    auto str1 = "hello";
    auto str2 = " world";

    auto cat = catStr(str1, str2);
    assert(cat == "hello world");
    Delete(cat);
}
